unit CalendarClock;
  (* replaces desktop gadgets *)

  interface
    uses
      Classes,
      ComCtrls (* tDateTimePicker, tMonthCalendar *),
      Controls,
      ExtCtrls (* tImage, tLabeledEdit, tTimer *),
      Forms    (* tCloseAction, tForm, Screen *),
      Jpeg     (* used by ExtCtrls *),
      StdCtrls (* tButton, tCheckBox, tGroupBox, tLabel *),
      Windows;

    type
      tfmCalendarClock = class(tForm)
          btnExpandContract: tButton;
          grpEvent: tGroupBox;
            ckbxSetAlarm: tCheckBox;
            dtpEventDate: tDateTimePicker;
            dtpEventTime: tDateTimePicker;
            lbedEvent: tLabeledEdit;
          imgAlarm: tImage;
          lbDay:       tLabel;
          lbDayOfWeek: tLabel;
          lbMonth:     tLabel;
          lbTime:      tLabel;
          Calendar: tMonthCalendar;
          Timer: tTimer;
          procedure btnExpandContractClick(Sender: tObject);
          procedure ckbxSetAlarmClick     (Sender: tObject);
          procedure dtpEventDateTimeChange(Sender: tObject);
          procedure FormShow              (Sender: tObject);
          procedure lbedEventChange       (Sender: tObject);
          procedure TimerTimer            (Sender: tObject);

          procedure FormClose(Sender: TObject; var Action: tCloseAction);
        private
          procedure Contract;
          procedure Expand;
        end (* tfmCalendarClock *);

    var
      fmCalendarClock: tfmCalendarClock;

  implementation
    uses
      DateUtils (* DayOf, DayOfTheWeek, HourOf, MinuteOf, MonthOf, RecodeTime *),
      Graphics  (* clBlue, clRed *),
      MyConsts  (* MINUS, nullstring, PLUS *),
      Registry  (* tRegistry *),
      SysUtils  (* Beep, Date, FormatDateTime, IntToStr, Now *);
      {$R *.dfm}

    const
      CreateIfMissing = true;
      ClockCalendarKey    = 'Software\RRF\ClockCalendar';
      AlarmDateTimeKey    = 'AlarmDateTime';
      EventKey            = 'event';

      CompactWidth  = 278;
      ExpandedWidth = 663;

    var
      AlarmDateTime: tDateTime;
      CompactHeight, ExpandedHeight, PrimaryWidth: integer;

    procedure tfmCalendarClock.btnExpandContractClick(Sender: tObject);
      (* change format of the form *)
    
      begin (* tfmCalendarClock.btnExpandContractClick *)
        if btnExpandContract.Caption = PLUS then
          Expand
        else
          Contract
      end   (* tfmCalendarClock.btnExpandContractClick *);

    procedure tfmCalendarClock.ckbxSetAlarmClick(Sender: tObject);
      (* alarm just set or unset *)
    
      begin (* tfmCalendarClock.ckbxSetAlarmClick *)
        imgAlarm.Visible := ckbxSetAlarm.Checked
      end   (* tfmCalendarClock.ckbxSetAlarmClick *);

    procedure tfmCalendarClock.Contract;
      (* minimize in UR corner of screen *)

      begin (* tfmCalendarClock.Contract *)
        Color := clRed;
        ClientHeight := CompactHeight;
        Top := 2;
        Width := CompactWidth;
        Left := PrimaryWidth - Width;
        btnExpandContract.Caption := PLUS
      end   (* tfmCalendarClock.Contract *);

    procedure tfmCalendarClock.dtpEventDateTimeChange(Sender: tObject);
      (* compute AlarmDateTime *)

      begin (* tfmCalendarClock.dtpEventDateTimeChange *)
        AlarmDateTime := RecodeTime(dtpEventDate.Date,
                                    HourOf(dtpEventTime.Time),
                                    MinuteOf(dtpEventTime.Time),
                                    0, 0)
      end   (* tfmCalendarClock.dtpEventDateTimeChange *);

    procedure tfmCalendarClock.Expand;
      (* show it all in UR corner of screen *)
    
      begin (* tfmCalendarClock.Expand *)
        ClientHeight := ExpandedHeight;
        Width := ExpandedWidth;
        Left := PrimaryWidth - Width;
        btnExpandContract.Caption := MINUS
      end   (* tfmCalendarClock.Expand *);

    procedure tfmCalendarClock.FormClose(Sender: tObject;
                                         var Action: tCloseAction);
      (* save the current alarm setting *)
    
      begin (* tfmCalendarClock.FormClose *)
        with tRegistry.Create do
          try     (* using TheRegistry *)
            OpenKey(ClockCalendarKey, CreateIfMissing);
            if ckbxSetAlarm.Checked then
              begin (* save the event *)
                WriteDateTime(AlarmDateTimeKey, AlarmDateTime);
                WriteString(EventKey, lbedEvent.Text)
              end   (* save the event *)
            else if not DeleteValue(AlarmDateTimeKey) then
              (* ignore it *)
          finally (* using TheRegistry *)
            Free
          end     (* using TheRegistry *)
      end   (* tfmCalendarClock.FormClose *);

    procedure tfmCalendarClock.FormShow(Sender: tObject);
      (* show current date & time *)

      const
        OneSecond = 1000;

      var
        I: integer;

      begin (* tfmCalendarClock.FormShow *)
        CompactHeight := Calendar.Top-1;
        ExpandedHeight := grpEvent.Top + grpEvent.Height + grpEvent.Left;
        PrimaryWidth := Screen.Width;
        for I := 1 to Screen.MonitorCount do
          with Screen.Monitors[I-1] do
            if Primary then
              PrimaryWidth := Width;

        dtpEventDate.Date := Date;
        dtpEventTime.Time := Now;
        AlarmDateTime := Now;
        ckbxSetAlarm.Checked := false;
        imgAlarm.Visible := false;

        with tRegistry.Create do
          try     (* using the registry *)
            if OpenKey(ClockCalendarKey, not CreateIfMissing) then
              if ValueExists(AlarmDateTimeKey) then
                begin (* use old alarm *)
                  AlarmDateTime := ReadDateTime(AlarmDateTimeKey);
                  lbedEvent.Text := ReadString(EventKey);
                  ckbxSetAlarm.Checked := true
                end   (* use old alarm *)
          finally (* using the registry *)
            Free
          end     (* using the registry *);

        Contract;
        btnExpandContract.SetFocus;
        Timer.Interval := 10 * OneSecond;
        Timer.Enabled := true;
        TimerTimer(nil)
      end   (* tfmCalendarClock.FormShow *);

    procedure tfmCalendarClock.lbedEventChange(Sender: tObject);
      (* update caption(s) *)
    
      begin (* tfmCalendarClock.lbedEventChange *)
        grpEvent.Caption := lbedEvent.Text
      end   (* tfmCalendarClock.lbedEventChange *);

    procedure tfmCalendarClock.TimerTimer(Sender: tObject);
      (* clock just ticked *)

      const
        Weekday: array[1 .. 7] of string =
          ('Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday');
        Month: array[1 .. 12] of string =
          ('January', 'February', 'March',     'April',   'May',      'June',
           'July',    'August',   'September', 'October', 'November', 'December');

      var
        TimeNow: tDateTime;

      begin (* tfmCalendarClock.TimerTimer *)
        TimeNow := Now;
        lbDay.Caption := IntToStr(DayOf(TimeNow));
        lbDayOfWeek.Caption := Weekday[DayOfTheWeek(TimeNow)];
        lbMonth.Caption := Month[MonthOf(TimeNow)];
        lbTime.Caption := FormatDateTime('hh:nn', TimeNow);
        Calendar.Date := Date;

        Caption := nullstring;
        if ckbxSetAlarm.Checked then
          if TimeNow >= AlarmDateTime then
            begin (* event *)
              Caption := lbedEvent.Text;
              Beep;
              SetFocus;
              Expand;
              Left := random(PrimaryWidth);
              if Color = clRed then
                Color := clBlue
              else
                Color := clRed
            end   (* event *)
      end   (* tfmCalendarClock.TimerTimer *);

end.
